searchData={"items":[{"type":"task","title":"mix dev.storybook","doc":"Make sure your storybook local dependency has all its assets packaged in priv.\n\n```bash\n$> mix dev.storybook\n```","ref":"Mix.Tasks.Dev.Storybook.html"},{"type":"task","title":"mix phx.gen.storybook","doc":"Generates a Storybook and provides setup instructions.\n\n```bash\n$> mix phx.gen.storybook\n```\n\nThe generated files will contain:\n\n  * a storybook backend in `lib/my_app_web/storybook.ex`\n  * a custom js file in `assets/js/storybook.js`\n  * a custom css file in `assets/css/storybook.css`\n  * scaffolding including example stories for your own storybook in `storybook/`\n\nThe generator supports the `--no-tailwind` flag if you want to skip the TailwindCSS specific bit.","ref":"Mix.Tasks.Phx.Gen.Storybook.html"},{"type":"module","title":"PhoenixStorybook","doc":"PhoenixStorybook provides a [_storybook-like_](https://storybook.js.org) UI interface for your\nPhoenix components.\n\n- Explore all your components, and showcase them with different variations.\n- Browse your component's documentation, with their supported attributes.\n- Learn how components behave by using an interactive playground.\n\n![screenshot](https://github.com/phenixdigital/phoenix_storybook/raw/main/screenshots/screenshot-01.jpg)\n![screenshot](https://github.com/phenixdigital/phoenix_storybook/raw/main/screenshots/screenshot-02.jpg)","ref":"PhoenixStorybook.html"},{"type":"module","title":"How does it work? - PhoenixStorybook","doc":"PhoenixStorybook is mounted in your application router and serves its UI at the mounting point of\nyour choice.\n\nIt performs automatic discovery of your storybook content under a specified folder (`:content_path`)\nand then automatically generates a storybook navigation sidebar. Every module detected in your\ncontent folder will be loaded and identified as a storybook entry.\n\nThree kinds of stories are supported:\n\n- `component` to describe your stateless function components or your live_components.\n- `page` to write & document UI guidelines, or whatever content you want.\n- `example` to show how your components can be used and mixed in real UI pages.","ref":"PhoenixStorybook.html#module-how-does-it-work"},{"type":"module","title":"Installation - PhoenixStorybook","doc":"To start using `PhoenixStorybook` in your phoenix application you will need to follow these steps:\n\n1. Add the `phoenix_storybook` dependency\n2. Run the generator","ref":"PhoenixStorybook.html#module-installation"},{"type":"module","title":"1. Add the `phoenix_storybook` dependency - PhoenixStorybook","doc":"Add the following to your mix.exs and run mix deps.get:\n\n```elixir\ndef deps do\n  [\n    {:phoenix_storybook, \"~> 0.7.0\"}\n  ]\nend\n```\n\n> [!IMPORTANT]\n>\n> When picking a github version of the library (instead of an official hex.pm release) you\n> need the get the storybook's assets compiled.\n> To do so, please run `mix dev.storybook`.","ref":"PhoenixStorybook.html#module-1-add-the-phoenix_storybook-dependency"},{"type":"module","title":"2. Run the generator - PhoenixStorybook","doc":"Run from the root of your application:\n\n```bash\n$> mix deps.get\n$> mix phx.gen.storybook\n```\n\nAnd you are ready to go!\n\nℹ️ If you prefer manual setup, please read the [setup guide](guides/setup.md).","ref":"PhoenixStorybook.html#module-2-run-the-generator"},{"type":"module","title":"Configuration - PhoenixStorybook","doc":"Of all config settings, only the `:otp_app`, and `:content_path` keys are mandatory.\n\n```elixir\n# lib/my_app_web/storybook.ex\ndefmodule MyAppWeb.Storybook do\n  use PhoenixStorybook,\n    # OTP name of your application.\n    otp_app: :my_app,\n\n    # Path to your storybook stories (required).\n    content_path: Path.expand(\"../storybook\", __DIR__),\n\n    # Path to your JS asset, which will be loaded just before PhoenixStorybook's own\n    # JS. It's mainly intended to define your LiveView Hooks in `window.storybook.Hooks`.\n    # Remote path (not local file-system path) which means this file should be served\n    # by your own application endpoint.\n    js_path: \"/assets/storybook.js\",\n\n    # Path to your components stylesheet.\n    # Remote path (not local file-system path) which means this file should be served\n    # by your own application endpoint.\n    css_path: \"/assets/storybook.css\",\n\n    # This CSS class will be put on storybook container elements where your own styles should\n    # prevail. See the `guides/sandboxing.md` guide for more details.\n    sandbox_class: \"my-app-sandbox\",\n\n    # Custom storybook title. Default is \"Live Storybook\".\n    title: \"My Live Storybook\",\n\n    # Theme settings.\n    # Each theme must have a name, and an optional dropdown_class.\n    # When set, a dropdown is displayed in storybook header to let the user pick a theme.\n    # The dropdown_class is used to render the theme in the dropdown and identify which current\n    # theme is active.\n    #\n    # The chosen theme key will be passed as an assign to all components.\n    # ex: <.component theme={:colorful}/>\n    #\n    # The chosen theme class will also be added to the `.psb-sandbox` container.\n    # ex:  ... \n    #\n    # If no theme has been selected or if no theme is present in the URL the first one is enabled.\n    themes: [\n      default: [name: \"Default\"],\n      colorful: [name: \"Colorful\", dropdown_class: \"text-pink-400\"]\n    ],\n\n    # Color mode settings. Defaults to false and 'dark'.\n    color_mode: true,\n    color_mode_sandbox_dark_class: \"dark\",\n\n    # If you want to use custom FontAwesome icons.\n    font_awesome_plan: :pro, # default value is :free\n    font_awesome_kit_id: \"foo8b41bar4625\",\n    font_awesome_rendering: :webfont, # default value is :svg\n\n    # Story compilation mode, can be either `:eager` or `:lazy`.\n    # It defaults to `:lazy` in dev environment, `:eager` in other environments.\n    #   - When eager: all .story.exs & .index.exs files are compiled upfront.\n    #   - When lazy: only .index.exs files are compiled upfront and .story.exs are compile when the\n    #     matching story is loaded in UI.\n    compilation_mode: :eager,\n\n    # If you want to see debugging logs for storybooks compilation, set this to `true`. Default: `false`\n    compilation_debug: true\n  ]\n```\n\nAll settings can be overridden from your config files.\n\n```elixir\n# config/config.exs\nconfig :my_app, MyAppWeb.Storybook,\n  content_path: \"overridden/content/path\"\n```\n\nℹ️ Learn more on theming components in the [theming guide](guides/theming.md), icons in the\n[icons](guides/icons.md) guide and color mode in the [color modes guide](guides/color_modes.md).","ref":"PhoenixStorybook.html#module-configuration"},{"type":"behaviour","title":"PhoenixStorybook.BackendBehaviour","doc":"Behaviour implemented by your backend module.","ref":"PhoenixStorybook.BackendBehaviour.html"},{"type":"callback","title":"PhoenixStorybook.BackendBehaviour.config/2","doc":"Returns a configuration value from your config.exs storybook settings.\n\n`key` is the config key\n`default` is an optional default value if no value can be fetched.","ref":"PhoenixStorybook.BackendBehaviour.html#c:config/2"},{"type":"callback","title":"PhoenixStorybook.BackendBehaviour.content_tree/0","doc":"Returns a precompiled tree of your storybook stories.","ref":"PhoenixStorybook.BackendBehaviour.html#c:content_tree/0"},{"type":"callback","title":"PhoenixStorybook.BackendBehaviour.find_entry_by_path/1","doc":"Returns an entry from its absolute storybook path (not filesystem).","ref":"PhoenixStorybook.BackendBehaviour.html#c:find_entry_by_path/1"},{"type":"callback","title":"PhoenixStorybook.BackendBehaviour.flat_list/0","doc":"Returns all the nodes (stoires & folders) of the storybook content tree as a flat list.","ref":"PhoenixStorybook.BackendBehaviour.html#c:flat_list/0"},{"type":"callback","title":"PhoenixStorybook.BackendBehaviour.leaves/0","doc":"Returns all the leaves (only stories) of the storybook content tree.","ref":"PhoenixStorybook.BackendBehaviour.html#c:leaves/0"},{"type":"callback","title":"PhoenixStorybook.BackendBehaviour.storybook_path/1","doc":"Retuns a storybook path from a story module.","ref":"PhoenixStorybook.BackendBehaviour.html#c:storybook_path/1"},{"type":"module","title":"PhoenixStorybook.Guides","doc":"This module is meant to be used from generated `welcome.story.exs` page.\nIt renders HTML markup from markdown guides located in the guides/folder.\n\nMarkup is precompiled because:\n- we don't want to force user application to embed Earmark\n- we don't want to put markdown guides in priv","ref":"PhoenixStorybook.Guides.html"},{"type":"module","title":"Examples - PhoenixStorybook.Guides","doc":"```elixir\nGuides.markup(\"components.md\")\nGuides.markup(\"icons.md\")\n```","ref":"PhoenixStorybook.Guides.html#module-examples"},{"type":"function","title":"PhoenixStorybook.Guides.markup/1","doc":"","ref":"PhoenixStorybook.Guides.html#markup/1"},{"type":"module","title":"PhoenixStorybook.Index","doc":"An index is an optional file you can create in every folder of your storybook content tree to\nimprove rendering of the storybook sidebar.\n\nThe index files can be used:\n  - to customize the folder itself: name, icon and opening status.\n  - to customize folder direct children (only stories): name and icon.\n\nIndexes must be created as `index.exs` files.\n\nRead the [icons](guides/icons.md) guide for more information on custom icon usage.","ref":"PhoenixStorybook.Index.html"},{"type":"module","title":"Usage - PhoenixStorybook.Index","doc":"```elixir\n# storybook/_components.index.exs\ndefmodule MyAppWeb.Storybook.Components do\n  use PhoenixStorybook.Index\n\n  def folder_name, do: \"My Components\"\n  def folder_icon, do: {:fa, \"icon\"}\n  def folder_open?, do: true\n\n  def entry(\"a_component\"), do: [name: \"My Component\"]\n  def entry(\"other_component\"), do: [name: \"Another Component\", icon: {:fa, \"icon\", :thin}]\nend\n```","ref":"PhoenixStorybook.Index.html#module-usage"},{"type":"macro","title":"PhoenixStorybook.Index.__using__/1","doc":"Convenience helper for using the functions above.","ref":"PhoenixStorybook.Index.html#__using__/1"},{"type":"module","title":"PhoenixStorybook.Rendering.CodeRenderer","doc":"Responsible for rendering your components code snippet, for a given\n`PhoenixStorybook.Variation`.\n\nUses the `Makeup` libray for syntax highlighting.","ref":"PhoenixStorybook.Rendering.CodeRenderer.html"},{"type":"function","title":"PhoenixStorybook.Rendering.CodeRenderer.render/1","doc":"Renders a component code snippet from a `RenderingContext`.\nReturns a `Phoenix.LiveView.Rendered`.","ref":"PhoenixStorybook.Rendering.CodeRenderer.html#render/1"},{"type":"function","title":"PhoenixStorybook.Rendering.CodeRenderer.render/2","doc":"","ref":"PhoenixStorybook.Rendering.CodeRenderer.html#render/2"},{"type":"function","title":"PhoenixStorybook.Rendering.CodeRenderer.render_component_source/1","doc":"Renders source of a component story.\nReturns a rendered HEEx template.","ref":"PhoenixStorybook.Rendering.CodeRenderer.html#render_component_source/1"},{"type":"function","title":"PhoenixStorybook.Rendering.CodeRenderer.render_source/2","doc":"","ref":"PhoenixStorybook.Rendering.CodeRenderer.html#render_source/2"},{"type":"module","title":"PhoenixStorybook.Rendering.ComponentRenderer","doc":"Responsible for rendering your function & live components.","ref":"PhoenixStorybook.Rendering.ComponentRenderer.html"},{"type":"function","title":"PhoenixStorybook.Rendering.ComponentRenderer.render/1","doc":"Renders a component from a `RenderingContext`.\nReturns a `Phoenix.LiveView.Rendered`.","ref":"PhoenixStorybook.Rendering.ComponentRenderer.html#render/1"},{"type":"function","title":"PhoenixStorybook.Rendering.ComponentRenderer.render/2","doc":"","ref":"PhoenixStorybook.Rendering.ComponentRenderer.html#render/2"},{"type":"module","title":"PhoenixStorybook.Rendering.RenderingContext","doc":"A struct holding all data needed by `ComponentRenderer` and `CodeRenderer` to render story\nvariations.","ref":"PhoenixStorybook.Rendering.RenderingContext.html"},{"type":"function","title":"PhoenixStorybook.Rendering.RenderingContext.build/5","doc":"","ref":"PhoenixStorybook.Rendering.RenderingContext.html#build/5"},{"type":"module","title":"PhoenixStorybook.Router","doc":"Provides LiveView routing for storybook.","ref":"PhoenixStorybook.Router.html"},{"type":"macro","title":"PhoenixStorybook.Router.live_storybook/2","doc":"Defines a PhoenixStorybook route.\n\nIt expects the `path` the storybook will be mounted at and a set of options.\n\nThis will also generate a named helper called `live_storybook_path/2` which you can use to link\ndirectly to the storybook, such as:\n\n```elixir\n<.link href={live_storybook_path(conn, :root)} />\n```\n\nNote that you should only use the `href` attribute to link to the storybook,\nas it has to set its own session on first rendering. Linking with `patch` or\n`navigate` will not work.","ref":"PhoenixStorybook.Router.html#live_storybook/2"},{"type":"macro","title":"Options - PhoenixStorybook.Router.live_storybook/2","doc":"* `:backend_module` - _Required_ - Name of your backend module.\n  * `:live_socket_path` - Configures the socket path. It must match the\n    `socket \"/live\", Phoenix.LiveView.Socket` in your endpoint.\n  * `:assets_path` - Configures the assets path. It must match the `storybook_assets` in your\n     router. Defaults to `\"/storybook/assets\"`.\n  * `:session_name` - Configures the live session name. Defaults to `:live_storybook`. Use this\n     option if you want to mount multiple storybooks in the same router.\n  * `:as` - Allows you to set the route helper name. Defaults to`:live_storybook`.\n  * `:pipeline` - Set to `false` if you don't want a router pipeline to be created. This is useful\n     if you want to define your own `:storybook_browser` pipeline, or if you mount multiple\n     storybooks, in which case the pipeline only has to be defined once. Defaults to `true`.\n  * `:csrf` - Set to `false` if your LiveView socket is configured without csrf, then the JS code\n     will attach to the socket without providing a csrf token. Defaults to `true`.","ref":"PhoenixStorybook.Router.html#live_storybook/2-options"},{"type":"macro","title":"Usage - PhoenixStorybook.Router.live_storybook/2","doc":"```elixir\n# lib/my_app_web/router.ex\nuse MyAppWeb, :router\nimport PhoenixStorybook.Router\n...\n\nscope \"/\" do\n  pipe_through :browser\n  live_storybook \"/storybook\", backend_module: MyAppWeb.Storybook\nend\n```\n\nNote that it is not possible to use this macro in a scope with a path\ndifferent from `/`.","ref":"PhoenixStorybook.Router.html#live_storybook/2-usage"},{"type":"macro","title":"PhoenixStorybook.Router.storybook_assets/1","doc":"Defines routes for PhoenixStorybook static assets.\n\nStatic assets should not be CSRF protected. So they need to be mounted in your\nrouter in a different pipeline than storybook's.\n\nIt can take the `path` the storybook assets will be mounted at.\nDefault path is `\"/storybook/assets\"`.","ref":"PhoenixStorybook.Router.html#storybook_assets/1"},{"type":"macro","title":"Usage - PhoenixStorybook.Router.storybook_assets/1","doc":"```elixir\n# lib/my_app_web/router.ex\nuse MyAppWeb, :router\nimport PhoenixStorybook.Router\n...\n\nscope \"/\" do\n  storybook_assets()\nend\n```","ref":"PhoenixStorybook.Router.html#storybook_assets/1-usage"},{"type":"module","title":"PhoenixStorybook.Stories.Attr","doc":"An attr is one of your component attributes. Its structure mimics the LiveView 0.18.0 declarative\nassigns.\n\nAttributes declaration will populate the Playground tab of your storybook, for each of your\ncomponents.","ref":"PhoenixStorybook.Stories.Attr.html"},{"type":"type","title":"PhoenixStorybook.Stories.Attr.t/0","doc":"- `id`: the attribute id (required). Should match your component assign.\n- `type`: the attribute type (required). Must be one of:\n  * `:any` - any term\n  * `:string` - any binary string\n  * `:atom` - any atom\n  * `:boolean` - any boolean\n  * `:integer` - any integer\n  * `:float` - any float\n  * `:map` - any map\n  * `:list` - a List of any arbitrary types\n  * `:global`- any common HTML attributes,\n  * Any struct module\n- `required`: `true` if the attribute is mandatory.\n- `default`: attribute default value.\n- `examples` the list or range of examples suggested for the attribute\n- `values` the list or range of all possible examples for the attribute. Unlike examples, this\n   option enforces validation of the default value against the given list.\n- `doc`: a text documentation for this attribute.","ref":"PhoenixStorybook.Stories.Attr.html#t:t/0"},{"type":"module","title":"PhoenixStorybook.Stories.Doc","doc":"Functions to fetch component documentation and render it at HTML.","ref":"PhoenixStorybook.Stories.Doc.html"},{"type":"function","title":"PhoenixStorybook.Stories.Doc.fetch_doc_as_html/1","doc":"Fetch component documentation from component source and format it as HTML.\n- For a live_component, fetches @moduledoc content\n- For a function component, fetches @doc content of the relevant function\n\nOutput HTML is splitted in paragraphs and returned as a list of paragraphs.","ref":"PhoenixStorybook.Stories.Doc.html#fetch_doc_as_html/1"},{"type":"module","title":"PhoenixStorybook.Stories.Slot","doc":"A slot is one of your component slots. Its structure mimics the LiveView 0.18.0 declarative\nassigns.\n\nSlots declaration will populate the Playground tab of your storybook, for each of your\ncomponents.\n\nSupported keys:\n- `id`: the slot id (required). Should match your component slot name.\n  Use the id `:inner_block` for your component default slot.\n- `doc`: a text documentation for this slot.\n- `required`: `true` if the attribute is mandatory.","ref":"PhoenixStorybook.Stories.Slot.html"},{"type":"type","title":"PhoenixStorybook.Stories.Slot.t/0","doc":"","ref":"PhoenixStorybook.Stories.Slot.html#t:t/0"},{"type":"module","title":"PhoenixStorybook.Stories.Variation","doc":"A variation captures the rendered state of a UI component. Developers write multiple variations\nper component that describe all the “interesting” states a component can support.\n\nEach variation will be displayed in the storybook as a code snippet alongside with the\ncomponent preview.\n\nVariations attributes type are checked against their matching attribute (if any) and will raise\na compilation an error in case of mismatch.\n\nAdvanced component & variation documentation is available in the\n[components guide](guides/components.md).","ref":"PhoenixStorybook.Stories.Variation.html"},{"type":"module","title":"Usage - PhoenixStorybook.Stories.Variation","doc":"```elixir\n  def variations do\n    [\n      %Variation{\n        id: :default,\n        description: \"Default dropdown\",\n        attributes: %{\n          label: \"A dropdown\",\n        },\n        slots: [\n          ~s|<:entry path=\"#\" label=\"Account settings\"/>|,\n          ~s|<:entry path=\"#\" label=\"Support\"/>|,\n          ~s|<:entry path=\"#\" label=\"License\"/>|\n        ]\n      }\n    ]\n  end\n```","ref":"PhoenixStorybook.Stories.Variation.html#module-usage"},{"type":"type","title":"PhoenixStorybook.Stories.Variation.t/0","doc":"","ref":"PhoenixStorybook.Stories.Variation.html#t:t/0"},{"type":"module","title":"PhoenixStorybook.Stories.VariationGroup","doc":"A variation group is a set of similar variations that will be rendered together in a single\npreview   block.","ref":"PhoenixStorybook.Stories.VariationGroup.html"},{"type":"module","title":"Usage - PhoenixStorybook.Stories.VariationGroup","doc":"```elixir\n  def variations do\n    [\n      %VariationGroup{\n        id: :colors,\n        description: \"Different color buttons\",\n        variations: [\n          %Variation{\n            id: :blue_button,\n            attributes: %{label: \"A button\", color: :blue }\n          },\n          %Variation{\n            id: :red_button,\n            attributes: %{label: \"A button\", color: :red }\n          },\n          %Variation{\n            id: :green_button,\n            attributes: %{label: \"A button\", color: :green }\n          }\n        ]\n      }\n    ]\n  end\n```","ref":"PhoenixStorybook.Stories.VariationGroup.html#module-usage"},{"type":"type","title":"PhoenixStorybook.Stories.VariationGroup.t/0","doc":"","ref":"PhoenixStorybook.Stories.VariationGroup.html#t:t/0"},{"type":"module","title":"PhoenixStorybook.Story","doc":"A story designates any kind of content in your storybook. For now only following kinds of stories\nare supported `:component`, `:live_component`, and `:page`.\n\nIn order to populate your storybook, just create _story_ scripts under your content path, and\nimplement their required behaviour.\n\nStories must be created as `story.exs` files.\n\nIn dev environment, stories are lazily compiled when reached from the UI.","ref":"PhoenixStorybook.Story.html"},{"type":"module","title":"Usage - PhoenixStorybook.Story","doc":"","ref":"PhoenixStorybook.Story.html#module-usage"},{"type":"module","title":"Component - PhoenixStorybook.Story","doc":"Implement your component as such.\nConfer to:\n- `PhoenixStorybook.Variation` documentation for variations.\n- `PhoenixStorybook.Attr` documentation for attributes.\n\n```elixir\n# storybook/my_component.exs\ndefmodule MyAppWeb.Storybook.MyComponent do\n  use PhoenixStorybook.Story, :component\n\n  # required\n  def function, do: &MyAppWeb.MyComponent.my_component/1\n\n  def attributes, do: []\n  def slots, do: []\n  def variations, do: []\nend\n```","ref":"PhoenixStorybook.Story.html#module-component"},{"type":"module","title":"Live Component - PhoenixStorybook.Story","doc":"Very similar to components, except that you need to define a `component/0` function instead of\n`function/0`.\n\n```elixir\n# storybook/my_live_component.exs\ndefmodule MyAppWeb.Storybook.MyLiveComponent do\n  use PhoenixStorybook.Story, :live_component\n\n  # required\n  def component, do: MyAppWeb.MyLiveComponent\n\n  def attributes, do: []\n  def slots, do: []\n  def variations, do: []\nend\n```\n\nℹ️ Learn more on components in the [components guide](guides/components.md).","ref":"PhoenixStorybook.Story.html#module-live-component"},{"type":"module","title":"Page - PhoenixStorybook.Story","doc":"A page is a fairly simple story that can be used to write whatever content you want. We use it to\nprovide some UI guidelines.\n\nYou should implement the render function and an optional navigation function, if you want a tab\nbased sub-navigation. Current tab is passed as `:tab` in `render/1` assigns.\n\n```elixir\n# storybook/my_page.exs\ndefmodule MyAppWeb.Storybook.MyPage do\n  use PhoenixStorybook.Story, :page\n\n  def doc, do: \"My page description\"\n\n  def navigation do\n    [\n      {:tab_one, \"Tab One\", {:fa, \"book\"}},\n      {:tab_two, \"Tab Two\", {:fa, \"cake\", :solid}}\n    ]\n  end\n\n  def render(assigns) do\n    ~H\" Your HEEX template \"\n  end\nend\n```","ref":"PhoenixStorybook.Story.html#module-page"},{"type":"module","title":"Example - PhoenixStorybook.Story","doc":"An example is a real-world UI showcasing how your components can be used and mixed in complex UI\ninterfaces.\n\nExamples are rendered as a child LiveView, so you can implement `mount/3`, `render/1` or any\n`handle_event/3` callback. Unfortunately `handle_params/3` cannot be defined in a child LiveView.\n\nBy default, your example story's source code will be shown in a dedicated tab. But you can show\nadditional files source code by implementing the `extra_sources/0` function which should return a\nlist of relative paths to your example related files.\n\n```elixir\n# storybook/my_example.story.exs\ndefmodule MyAppWeb.Storybook.MyPage do\n  use PhoenixStorybook.Story, :example\n\n  def doc, do: \"My page description\"\n\n  def extra_sources do\n    [\n      \"./template.html.heex\",\n      \"./my_page_html.ex\"\n    ]\n  end\n\n  def mount(_, _, socket), do: {:ok, socket}\n\n  def render(assigns) do\n    ~H\" Your HEEX template \"\n  end\nend\n```","ref":"PhoenixStorybook.Story.html#module-example"},{"type":"macro","title":"PhoenixStorybook.Story.__using__/1","doc":"Convenience helper for using the functions above.","ref":"PhoenixStorybook.Story.html#__using__/1"},{"type":"extras","title":"Color modes","doc":"# Color modes\n\nThe storybook supports three color modes: _dark_, _light_ and _system_.\n\n- The Storybook's styling adapts based on the selected color mode.\n- Your components are wrapped in a ` ` with a custom dark class.\n\nThe different modes are handled as follows:\n\n- `dark`: the `dark` class (or custom dark class) is applied to your component's sandbox\n- `light`: no class is applied\n- `system`: The `dark` class is added if your system prefers dark mode (as determined by the `prefers-color-scheme` media query).","ref":"color_modes.html"},{"type":"extras","title":"Setup - Color modes","doc":"To enable color mode support, you need to configure it in your Storybook setup:\n\n```elixir\nuse PhoenixStorybook,\n  # ...\n  color_mode: true\n```\n\nThis configuration adds a color theme picker to the Storybook header, allowing you to render the Storybook with the selected mode.","ref":"color_modes.html#setup"},{"type":"extras","title":"Component rendering - Color modes","doc":"When your components are rendered in Storybook, they are wrapped in a sandbox element (read [sandboxing guide](sandboxing.md)).\n\n- If the current color mode is dark (or system mode with dark preference), the sandbox will have a `dark` CSS class.\n- In light mode, no class is applied.\n\nYou can customize the default dark class by specifying it in your configuration:\n\n```elixir\nuse PhoenixStorybook,\n  # ...\n  color_mode_sandbox_dark_class: \"my-dark\",\n```","ref":"color_modes.html#component-rendering"},{"type":"extras","title":"Tailwind setup - Color modes","doc":"If you use Tailwind for your components, update your `tailwind.config.js` file as follows:\n\n```js\nmodule.exports = {\n  // ...\n  darkMode: \"selector\",\n};\n```\n\nTo use a custom dark class, modify the configuration like this:\n\n```js\nmodule.exports = {\n  // ...\n  darkMode: [\"selector\", \".my-dark\"],\n};\n```\n\nIn your application, ensure that the dark mode class is applied to your DOM element, particularly on or under your sandbox element:\n\n```html\n \n  ...\n \n```","ref":"color_modes.html#tailwind-setup"},{"type":"extras","title":"Component stories","doc":"# Component stories\n\nBasic component documentation is in `PhoenixStorybook.Story`.","ref":"components.html"},{"type":"extras","title":"Documentation - Component stories","doc":"Component documentation is fetched from your component doc tags:\n\n- For a live_component, fetches `@moduledoc` content.\n- For a function component, fetches `@doc` content from the matching function.\n\nIf you are deploying `phoenix_storybook` in production with an Elixir release, make sure your\ndoc chunks are not [stripped out from the release.](https://hexdocs.pm/mix/Mix.Tasks.Release.html#module-customization)\n\n```elixir\nreleases: [\n  my_app_web: [\n    strip_beams: [\n      keep: [\"Docs\"]\n    ]\n  ]\n]\n```","ref":"components.html#documentation"},{"type":"extras","title":"Variation groups - Component stories","doc":"You may want to present different variations of a component in a single variation block.\nIt is possible using `PhoenixStorybook.VariationGroup`.","ref":"components.html#variation-groups"},{"type":"extras","title":"Container - Component stories","doc":"By default, each `variation` is rendered within a `div` in the storybook DOM.\nYou can pass additional HTML attributes or extend the class attribute.\n\n```elixir\n# storybook/my_component.story.exs\ndefmodule Storybook.MyComponent do\n  use PhoenixStorybook.Story, :component\n  def container, do: {:div, class: \"block\"}\nend\n```\n\nIf you need further _sandboxing_ you can opt in for `iframe` rendering.\n\n- For function components, storybook will use the iframe srcdoc attribute (the whole iframe content\n  is inlined as an HTML attribute).\n- For live components, storybook will use the typical iframe behavior, triggering an extra HTTP\n  request to fetch the iframe content.\n\n```elixir\n# storybook/my_component.story.exs\ndefmodule Storybook.MyComponent do\n  use PhoenixStorybook.Story, :component\n  def container, do: :iframe\n  # or def container, do: {:iframe, style: \"display: inline; ...\"}\nend\n```\n\nℹ️ Learn more on this topic in the [sandboxing guide](sandboxing.md).","ref":"components.html#container"},{"type":"extras","title":"Aliases & Imports - Component stories","doc":"When using nested components or JS commands, you might need to reference other functions or\ncomponents. Whilst it is possible to use fully qualified module names, you might want to provide\ncustom _aliases_ and _imports_.\n\nHere is an example defining both:\n\n```elixir\ndefmodule NestedComponent do\n  use PhoenixStorybook.Story, :component\n  def function, do: &NestedComponent.nested_component/1\n\n  def aliases, do: [MyStorybook.Helpers.JSHelpers]\n  def imports, do: [{NestedComponent, nested: 1}]\n\n  def variations do\n    [\n      %Variation{\n        id: :default,\n        slots: [\n          \"\"\"\n          <.nested phx-click={JSHelpers.toggle()}>hello</.nested>\n          <.nested phx-click={JSHelpers.toggle()}>world</.nested>\n          \"\"\"\n        ]\n      }\n    ]\n  end\nend\n```","ref":"components.html#aliases-imports"},{"type":"extras","title":"Templates - Component stories","doc":"You may want to render your components within some wrapping markup. For instance, when your\ncomponent can only be used as a slot of another enclosing component.\n\nSome components, such as _modals_, _slideovers_, and _notifications_, are not visible from the\nstart: they first need user interaction. Such components can be accompanied by an outer template,\nthat will for instance render a button next to the component, to toggle its visibility state.","ref":"components.html#templates"},{"type":"extras","title":"Variation templates - Component stories","doc":"You can define a template in your component story by defining a `template/0` function.\nEvery variation will be rendered within the defined template, the variation itself is injected\nin place of `<.psb-variation/>`.\n\n```elixir\ndef template do\n  \"\"\"\n   \n    <.psb-variation/>\n   \n  \"\"\"\nend\n```\n\nYou can also override the template, per variation or variation_group by setting the `:template` key\nto your variation. Setting it to a falsy value will disable templating for this variation.","ref":"components.html#variation-templates"},{"type":"extras","title":"Variation group templates - Component stories","doc":"Variation groups can also leverage on templating:\n\n- either by wrapping every variation in their own template.\n\n```elixir\n\"\"\"\n \n  <.psb-variation/>\n \n\"\"\"\n```\n\n- or by wrapping all variations as a whole, in a single template.\n\n```elixir\n\"\"\"\n \n  <.psb-variation-group/>\n \n\"\"\"\n```\n\nIf you want to get unique id, you can use `:variation_id` that will be replaced, at rendering time\nby the current variation (or variation group) id.","ref":"components.html#variation-group-templates"},{"type":"extras","title":"Placeholder attributes - Component stories","doc":"In template, you can pass some extra attributes to your variation. Just add them to the\n`.psb-variation` or `.psb-variation-group` placeholder.\n\n```elixir\n\"\"\"\n<.form_for :let={f} for={%{}} as={:user}>\n  <.psb-variation form={f}/>\n</.form>\n\"\"\"\n```","ref":"components.html#placeholder-attributes"},{"type":"extras","title":"JS-controlled visibility - Component stories","doc":"Here is an example of templated component managing its visibility client-side, by toggling CSS\nclasses through [JS commands](https://hexdocs.pm/phoenix_live_view/Phoenix.LiveView.JS.html).\n\n```elixir\ndefmodule Storybook.Components.Modal do\n  use PhoenixStorybook.Story, :component\n\n  def function, do: &Components.Modal.modal/1\n\n  def template do\n    \"\"\"\n     \n       Open modal \n      <.psb-variation/>\n     \n    \"\"\"\n  end\n\n  def variations do\n    [\n      %Variation{\n        id: :default_modal,\n        slots: [\"<:body>hello world</:body>\"]\n      }\n    ]\n  end\nend\n```","ref":"components.html#js-controlled-visibility"},{"type":"extras","title":"Elixir-controlled visibility - Component stories","doc":"Some components don't rely on JS commands but need external assigns, like a modal that takes a\n`show={true}` or `show={false}` assign to manage its visibility state.\n\n`PhoenixStorybook` handles special `psb-assign` and `psb-toggle` events that you\ncan leverage on to update some properties that will be passed to your components as _extra assigns_.\n\n```elixir\ndefmodule Storybook.Components.Slideover do\n  use PhoenixStorybook.Story, :component\n  def function, do: &Components.Slideover.slideover/1\n\n  def template do\n    \"\"\"\n     \n       \n        Open slideover\n       \n      <.psb-variation/>\n     \n    \"\"\"\n  end\n\n  def variations do\n    [\n      %Variation{\n        id: :default_slideover,\n        attributes: %{\n          close_event: JS.push(\"psb-assign\", value: %{variation_id: :default_slideover, show: false})\n        },\n        slots: [\"<:body>Hello world</:body>\"]\n      }\n    ]\n  end\nend\n```","ref":"components.html#elixir-controlled-visibility"},{"type":"extras","title":"Template code preview - Component stories","doc":"By default, the code preview will render the variation and its template markup as well.\nYou can choose to render only the variation markup, without its surrounding template by using the\n`psb-code-hidden` HTML attribute.\n\n```elixir\n\"\"\"\n \n   Open modal \n  <.psb-variation/>\n \n\"\"\"\n```","ref":"components.html#template-code-preview"},{"type":"extras","title":"Block, slots & let - Component stories","doc":"Liveview let you define blocks of HEEx content in your components, referred to as as slots.\nThey can be passed in your variations with the `:slots` keys :\n\n```elixir\n%Variation{\n  id: :modal,\n  slots: [\n    \"\"\"\n    <:button>\n       Cancel \n    </:button>\n    \"\"\",\n    \"\"\"\n    <:button>\n       OK \n    </:button>\n    \"\"\"\n  ]\n}\n```\n\nYou can also use [LiveView let mechanism](https://hexdocs.pm/phoenix_live_view/Phoenix.Component.html#module-default-slots)\nto pass data to your default block. You just need to **declare the let attribute** you are using in\nyour variation.\n\n```elixir\n%Variation{\n  id: :list,\n  attributes: %{stories: ~w(apple banana cherry)},\n  let: :entry,\n  slots: [\n    \"I like <%= entry %>\"\n  ]\n}\n```\n\n`let` syntax can also be used with named slots, but requires no specific livebook setup.\n\n```elixir\n%Variation{\n  id: :table,\n  attributes: %{\n    rows: [\n      %{first_name: \"Jean\", last_name: \"Dupont\"},\n      %{first_name: \"Sam\", last_name: \"Smith\"}\n    ]\n  },\n  slots: [\n    \"\"\"\n    <:col :let={user} label=\"First name\">\n      <%= user.first_name %>\n    </:col>\n    \"\"\",\n    \"\"\"\n    <:col :let={user} label=\"Last name\">\n      <%= user.last_name %>\n    </:col>\n    \"\"\"\n  ]\n}\n```","ref":"components.html#block-slots-let"},{"type":"extras","title":"Late evaluation - Component stories","doc":"In some cases, you want to pass to your variation attributes a complex value which should be\nevaluated at runtime but not in code preview (where you rather want to see the orignal expression).\n\nFor instance with the following variation of a `Modal` component.\n\n```elixir\n%Variation{\n  attributes: %{\n    :\"on-open\": JS.push(\"open\"),\n    :\"on-close\": {:eval, ~s|JS.push(\"close\")|}\n  }\n}\n```\n\nBoth open & close events would work, but code would be rendered like this.\n\n```\n<.modal\n  on-open=\"%Phoenix.LiveView.JS{ops: [[\"push\", %{event: \"open\"}]]}\"\n  on-close={JS.push(\"close\")}\n/>\n```","ref":"components.html#late-evaluation"},{"type":"extras","title":"Layout - Component stories","doc":"You can control how your story variations are rendered in the stories tab by defining an optional `layout/0` function in any of your `component` or `live_component` stories.\n\nBy default a story will be rendered with the `:two_columns` layout. But you can use the alternate `:one_column` layout making the component preview taking the full container width.\n\n```elixir\ndefmodule Storybook.Components.Breadcrumb do\n  use PhoenixStorybook.Story, :component\n\n  def layout, do: :one_column\n\n  # ...\nend\n```","ref":"components.html#layout"},{"type":"extras","title":"Custom Icons","doc":"# Custom Icons\n\nYou can provide custom sidebar & header icons for your stories.\nPage stories can also provide iconized navigation tabs.\n\nPhoenixStorybook gives you the ability to render icons with 2 different providers:\n\n- [FontAwesome](https://fontawesome.com) which offers a decent set of free icons and a lot of\n  additional styles with paid plans\n- [HeroIcons](https://heroicons.com) wich offer hundreds of free high quality icons\n- **Local** which offers the ability to use locally defined icons within the application","ref":"icons.html"},{"type":"extras","title":"Declaring an icon - Custom Icons","doc":"Whenever you provide an icon to the storybook, you should follow the following structure:\n\n```elixir\n{icon_provider, icon_name, :icon_style, additional_css_classes}\n\n# or for local icons, which don't support :icon_style\n{icon_provider, icon_name, additional_css_classes}\n```\n\nThe two last tuple elements are optional.\n\n```elixir\n{:fa, \"book\"} # note that the FontAwesome icon name omits the fa- prefix\n{:fa, \"book\", :solid} # same than previous one, :solid is the default style\n{:fa, \"skull\", :duotone} # only for FontAwesome paid users\n{:fa, \"skull\", :duotone, \"psb-px-2\"}\n{:hero, \"cake\"} # for all HeroIcons\n{:hero, \"cake\", :outline} # same than previous one, :outline is the default style\n{:hero, \"cake\", :outline, \"psb-w-2 psb-h-2\"}\n{:local, \"hero-cake\"} # for custom or core-component-generated icons\n{:local, \"hero-cake\", \"psb-w-2 psb-h-2\"}, # styles are not supported: 3rd elem is custom css\n```","ref":"icons.html#declaring-an-icon"},{"type":"extras","title":"FontAwesome icons - Custom Icons","doc":"PhoenixStorybook is built with a very small subset of FontAwesome free icons. If you want to use\nother FontAwesome icons you need to provide a valid **FontAwesome kit id**.\n\nWhen configuring your kit, please make sure you pick _Web Fonts_ & _CSS Only_ options.\n\nIt can be either free or paid, so you also need to configure your FontAwesome plan.\n\n```elixir\n# lib/my_app_web/storybook.ex\ndefmodule MyAppWeb.Storybook do\n  use PhoenixStorybook,\n    otp_app: :my_app,\n    font_awesome_plan: :pro, # default value is :free\n    font_awesome_kit_id: \"foo8b41bar4625\",\nend\n```","ref":"icons.html#fontawesome-icons"},{"type":"extras","title":"HeroIcons - Custom Icons","doc":"HeroIcons are supported in two flavors: by using [heroicons_elixir](https://github.com/mveytsman/heroicons_elixir)\nor directly using [heroicons CSS](https://github.com/tailwindlabs/heroicons) resources.","ref":"icons.html#heroicons"},{"type":"extras","title":"heroicons_elixir - Custom Icons","doc":"Add heroicons dependency in your `mix.exs` file.\n\n```elixir\ndefp deps do\n  [\n    {:heroicons, \"~> 0.5.0\"}\n  ]\nend\n```\n\nYou can now use whichever HeroIcon icon you want, based on the library function names.\n\n```elixir\n{:hero, \"cake\"} # for all HeroIcons\n{:hero, \"cake\", :outline} # same than previous one, :outline is the default style\n{:hero, \"cake\", :outline, \"psb-w-2 psb-h-2\"}\n```","ref":"icons.html#heroicons_elixir"},{"type":"extras","title":"heroicons CSS - Custom Icons","doc":"[Phoenix framework now encourages the use of Heroicons](https://github.com/phoenixframework/phoenix/pull/5634)\nas raw CSS classes rather than relying on an Elixir package.\n\nPut `tailwindlabs/heroicons` dependency in your mix file:\n\n```elixir\ndefp deps do\n  [\n    {\n      :heroicons,\n      github: \"tailwindlabs/heroicons\",\n      tag: \"v2.2.0\",\n      sparse: \"optimized\",\n      app: false,\n      compile: false,\n      depth: 1\n    }\n  ]\nend\n```\n\nand declare a plugin in `tailwind.config.js`:\n\n```js\nplugins: [\n  plugin(function ({ matchComponents, theme }) {\n    let iconsDir = path.join(__dirname, \"../deps/heroicons/optimized\");\n    let values = {};\n    let icons = [\n      [\"\", \"/24/outline\"],\n      [\"-solid\", \"/24/solid\"],\n      [\"-mini\", \"/20/solid\"],\n      [\"-micro\", \"/16/solid\"],\n    ];\n    icons.forEach(([suffix, dir]) => {\n      fs.readdirSync(path.join(iconsDir, dir)).forEach((file) => {\n        let name = path.basename(file, \".svg\") + suffix;\n        values[name] = { name, fullPath: path.join(iconsDir, dir, file) };\n      });\n    });\n    matchComponents(\n      {\n        hero: ({ name, fullPath }) => {\n          let content = fs\n            .readFileSync(fullPath)\n            .toString()\n            .replace(/\\r?\\n|\\r/g, \"\");\n          let size = theme(\"spacing.6\");\n          if (name.endsWith(\"-mini\")) {\n            size = theme(\"spacing.5\");\n          } else if (name.endsWith(\"-micro\")) {\n            size = theme(\"spacing.4\");\n          }\n          return {\n            [`--hero-${name}`]: `url('data:image/svg+xml;utf8,${content}')`,\n            \"-webkit-mask\": `var(--hero-${name})`,\n            mask: `var(--hero-${name})`,\n            \"mask-repeat\": \"no-repeat\",\n            \"background-color\": \"currentColor\",\n            \"vertical-align\": \"middle\",\n            display: \"inline-block\",\n            width: size,\n            height: size,\n          };\n        },\n      },\n      { values }\n    );\n  }),\n],\n```\n\nThen you can use Heroicons as local icons:\n\n```elixir\n{:local, \"hero-cake\"} # for custom or core-component-generated icons\n{:local, \"hero-cake\", \"psb-w-2 psb-h-2\"}, # styles are not supported: 3rd\n```","ref":"icons.html#heroicons-css"},{"type":"extras","title":"Local - Custom Icons","doc":"Use whichever icons you want, as long as they can be rendered via a span class tag.\n\n```elixir\n{:local, \"my-icon\", \"psb-w-2 psb-h-2\"}\n```\n\nwill be rendered as:\n\n```html\n  \n```","ref":"icons.html#local"},{"type":"extras","title":"Custom CSS - Custom Icons","doc":"The last tuple argument is an optional CSS class list you can pass to improve icon rendering.\nSince the icon is rendered within the storybook layout, and not within your components sandbox, you\nshould use CSS classes supported by the storybook.\n\n- any `psb-w-*` or `psb-h-*` class (TailwindCSS classes for height & width prefixed by `psb-`)\n- any `psb-p-*` or `psb-m-*` class (padding & margin)\n- any `psb-text-color-###`\n- any `fa-*` (FontAwesome modifiers)","ref":"icons.html#custom-css"},{"type":"extras","title":"Sandboxing components","doc":"# Sandboxing components\n\nIn `PhoenixStorybook` your components live within the storybook, so they share some context with\nthe storybook: **styling** and **scripts**.\n\nWhile the original Storybook for React only [relies on iframes](https://storybook.js.org/docs/react/configure/story-rendering),\nwe find them quite slow and don't want them to be the default choice.\n\nThis guide will explain:\n\n- what JS context do your components share with the storybook?\n- how is the storybook styled to prevent most styling clashes?\n- how you should provide the style of your components with scoped styles.\n- how to, as a last resort, enable iframe rendering.","ref":"sandboxing.html"},{"type":"extras","title":"What JS context do your components share with the storybook? - Sandboxing components","doc":"`PhoenixStorybook` runs with Phoenix LiveView and therefore requires its `LiveSocket`. This\nLiveSocket is the same used by your components: you just need to inject it with your own `Hooks`,\n`Params` and `Uploaders`.\n\nTo do so, create a JS file that will declare your `Hooks`, `Params` and `Uploaders` and set them in\n`window.storybook`. This script will be loaded immediately before the storybook's script.\n\n> :information_source: If you used `mix phx.gen.storybook` this file has already been created for you.\n\n```javascript\n// assets/js/storybook.js\nimport * as Hooks from \"./hooks\";\nimport * as Params from \"./params\";\nimport * as Uploaders from \"./uploaders\";\n(function () {\n  window.storybook = { Hooks, Params, Uploaders };\n})();\n```\n\nThen set the `js_path: \"/assets/storybook.js\"` option to the storybook within your `storybook.ex`\nfile. This is a remote path (not a local file-system path) which means this file should be served\nby your own application endpoint with the given path.\n\nYou can also use this script to inject whatever content you want into document `HEAD`, such as\nexternal scripts.\n\nThe `Params` will be available in page stories as `connect_params` assign.\nThere is currently no way to access them in component or live component stories.","ref":"sandboxing.html#what-js-context-do-your-components-share-with-the-storybook"},{"type":"extras","title":"How is the storybook styled? - Sandboxing components","doc":"`PhoenixStorybook` is using [TailwindCSS](https://tailwindcss.com) with\n[preflight](https://tailwindcss.com/docs/preflight) (which means all default HTML styles from your\nbrowser are removed) and a [custom prefix](https://tailwindcss.com/docs/configuration#prefix):\n`psb-` (which means that instead of using `bg-blue-400` the storybook uses `psb-bg-blue-400`).\n\nOnly elements with the `.psb` class are preflighted, in order to let your component styling as-is.\n\nSo unless your components use `psb` or `psb-` prefixed classes there should be no styling leak from\nthe storybook to you components.","ref":"sandboxing.html#how-is-the-storybook-styled"},{"type":"extras","title":"How should you provide the style of your components? - Sandboxing components","doc":"You need to inject your component's stylesheets into the storybook. Set the\n`css_path: \"/assets/storybook.css\"` option in `storybook.ex`. This is a remote path (not a local\nfile-system path) which means this file should be served by your own application endpoint with the\ngiven path.\n\nThe previous part was about storybook styles not leaking into your components. This part is\nabout the opposite: don't accidentally mess up Storybook styling with your styles.\n\nAll containers rendering your components in the storybook (`stories`, `playground`, `pages` ...)\ncarry the `.psb-sandbox` CSS class and a **custom sandboxing class of your choice**.\n\nYou can leverage this to scope your styles with this class. Here is how you can do it with\n`TailwindCSS`:\n\n- configure `phoenix_storybook` with a custom `sandbox_class`:\n\n```elixir\n# lib/my_app_web/storybook.ex\ndefmodule MyAppWeb.Storybook do\n  use PhoenixStorybook,\n    ...\n    sandbox_class: \"my-app-sandbox\",\n```\n\n- use Tailwind [important selector strategy](https://tailwindcss.com/docs/configuration#selector-strategy)\n  with this class. It will prefix all your tailwind classes increasing their specificity, hence\n  their priority.\n\n```javascript\n// assets/tailwind.config.js\nmodule.exports = {\n  // ...\n  important: \".my-app-sandbox\",\n};\n```\n\n- nest your custom styles under Tailwind `@layer utilities`. This way, your styling will also\n  benefit from sandboxing.\n\n```css\n/* assets/css/storybook.css */\n@tailwind base;\n@tailwind components;\n@tailwind utilities;\n\n@layer utilities {\n  /* this style will be generated as .my-app-sandbox * { ... } */\n  * {\n    font-family: \"MyComponentsFont\";\n    @apply text-slate-600;\n  }\n\n  /* this style will be generated as .my-app-sandbox h1 { ... } */\n  h1 {\n    @apply text-2xl font-bold text-slate-700 mt-2 mb-6;\n  }\n\n  /* this style will be generated as .my-app-sandbox h2 { ... } */\n  h2 {\n    @apply text-xl font-bold text-slate-700 mt-2 mb-4;\n  }\n}\n```","ref":"sandboxing.html#how-should-you-provide-the-style-of-your-components"},{"type":"extras","title":"Enabling iframe rendering - Sandboxing components","doc":"As a last resort, if for whatever reason you cannot make your component live within the storybook,\nit is possible to enable iframe rendering, component per component.\n\nThis could be required e.g. if you need to bind listeners on `document` or when\nyou want to make sure responsive css works as expected.\n\nJust add the `iframe` option to it.\n\n```elixir\n# storybook/components/button.exs\ndefmodule MyAppWeb.Storybook.Components.Button do\n alias MyAppWeb.Components.Button\n use PhoenixStorybook.Story, :component\n\n def function, do: &Button.button/1\n def container, do: :iframe\n\n # ...\nend\n```","ref":"sandboxing.html#enabling-iframe-rendering"},{"type":"extras","title":"Manual setup","doc":"# Manual setup\n\nTo start using `PhoenixStorybook` in your Phoenix application you will need to follow these steps:\n\n1. Add the `phoenix_storybook` dependency\n2. Create your storybook backend module\n3. Add storybook access to your router\n4. Make your components' assets available\n5. Update your Docker image\n6. Create some content","ref":"setup.html"},{"type":"extras","title":"1. Add the `phoenix_storybook` dependency - Manual setup","doc":"Add the following to your mix.exs and run mix deps.get:\n\n```elixir\ndef deps do\n  [\n    {:phoenix_storybook, \"~> 0.7.0\"}\n  ]\nend\n```","ref":"setup.html#1-add-the-phoenix_storybook-dependency"},{"type":"extras","title":"2. Create your storybook backend module - Manual setup","doc":"Create a new module under your application lib folder:\n\n```elixir\n# lib/my_app_web/storybook.ex\ndefmodule MyAppWeb.Storybook do\n  use PhoenixStorybook,\n    otp_app: :my_app,\n    content_path: Path.expand(\"../storybook\", __DIR__),\n    # assets path are remote path, not local file-system paths\n    css_path: \"/assets/my_components.css\",\n    js_path: \"/assets/my_components.js\"\nend\n```","ref":"setup.html#2-create-your-storybook-backend-module"},{"type":"extras","title":"3. Add storybook access to your router - Manual setup","doc":"Once installed, update your router's configuration to forward requests to a `PhoenixStorybook`\nwith a unique name of your choice:\n\n```elixir\n# lib/my_app_web/router.ex\nuse MyAppWeb, :router\nimport PhoenixStorybook.Router\n...\nscope \"/\" do\n  storybook_assets()\nend\n\nscope \"/\", PhoenixStorybookSampleWeb do\n  pipe_through(:browser)\n  ...\n  live_storybook \"/storybook\", backend_module: MyAppWeb.Storybook\nend\n```","ref":"setup.html#3-add-storybook-access-to-your-router"},{"type":"extras","title":"4. Make your components' assets available - Manual setup","doc":"Build a new CSS bundle dedicated to your live_view components: this bundle will be used both by your\napp and the storybook.\n\nIn this README, we use `assets/css/storybook.css` as an example.\n\nIf your components require any hooks or custom uploaders, or if your pages require connect parameters,\ndeclare them as such in a new JS bundle:\n\n```javascript\n// assets/js/storybook.js\n\nimport * as Hooks from \"./hooks\";\nimport * as Params from \"./params\";\nimport * as Uploaders from \"./uploaders\";\n\n(function () {\n  window.storybook = { Hooks, Params, Uploaders };\n})();\n```\n\nYour application must bundle these assets and serve them. Our custom `mix phx.gen.storybook`\ngenerator may guide you through these steps.\n\nℹ️ Learn more on this topic in the [sandboxing guide](guides/sandboxing.md).","ref":"setup.html#4-make-your-components-assets-available"},{"type":"extras","title":"5. Update your Docker image - Manual setup","doc":"If you are deploying your app with Docker, then you need to copy the storybook content into your\nDocker image.\n\nAdd this to your `Dockerfile`:\n\n```docker\nCOPY storybook storybook\n```","ref":"setup.html#5-update-your-docker-image"},{"type":"extras","title":"6. Create some content - Manual setup","doc":"Then you can start creating some content for your storybook. Storybook can contain different kinds\nof _stories_:\n\n- **component stories**: to document and showcase your components across different variations.\n- **pages**: to publish some UI guidelines, framework with regular HTML content.\n- **examples**: to show how your components can be used and mixed in real UI pages.\n\nStories are described as Elixir scripts (`.story.exs`) created under your `:content_path` folder.\nFeel free to organize them in sub-folders, as the hierarchy will be respected in your storybook\nsidebar.\n\nHere is an example of a stateless (function) component story:\n\n```elixir\n# storybook/components/button.story.exs\ndefmodule MyAppWeb.Storybook.Components.Button do\n  alias MyAppWeb.Components.Button\n\n  # :live_component or :page are also available\n  use PhoenixStorybook.Story, :component\n\n  def function, do: &Button.button/1\n\n  def variations do [\n    %Variation{\n      id: :default,\n      attributes: %{\n        label: \"A button\"\n      }\n    },\n    %Variation{\n      id: :green_button,\n      attributes: %{\n        label: \"Still a button\",\n        color: :green\n      }\n    }\n  ]\n  end\nend\n```","ref":"setup.html#6-create-some-content"},{"type":"extras","title":"Visual Regression Testing","doc":"# Visual Regression Testing\n\nWhile we still encourage you writing regular unit test for your components, this doesn't protect\nyou against visual regressions.\n\nVisual Regression Testing consists in taking automated screenshots of your components and compare\nthem pixel-per-pixel to notice any unwanted change.\n\nFor this we recommend using a dedicated tool such as [percy.io](https://percy.io/).\n\nThis library provides you a dedicated enpoint to output your stories (only components' stories)\nwithout the storybook main UI:\n\n- single story endpoint: `https://localhost:4000/storybook/visual_tests/buttons/button`\n- range story endpoint: `https://localhost:4000/storybook/visual_tests?start=a&end=e`\n\n_The last one renders all stories whose name starting between letter 'a' and letter 'e')_","ref":"testing.html"},{"type":"extras","title":"Theming components","doc":"# Theming components","ref":"theming.html"},{"type":"extras","title":"Theming Strategies - Theming components","doc":"The storybook gives you different possibilities to apply a theme to your components. These\npossibilities are named _strategies_.\n\nThe following strategies are available:\n\n1. _sandbox class_: set your theme as a CSS class, on the sandbox container, with a custom prefix\n2. _assign_: pass the theme as an assign to your components, with a custom key.\n3. _function_: call a custom module/function along with the current theme.\n\nHere is how you can use these strategies. In your `storybook.ex`:\n\n```elixir\nuse PhoenixStorybook,\n  themes_strategies: [\n    sandbox_class: \"prefix\", # will set a class prefixed by `prefix-` on the sandbox container\n    assign: :theme,\n    function: {MyApp.ThemeHelper, :register_theme}\n  ]\n```\n\nIf the `theme_strategies` key is undefined, the default `sandbox_class: \"theme\"` strategy is applied.","ref":"theming.html#theming-strategies"},{"type":"extras","title":"CSS theming - Theming components","doc":"By default, the storybook is applying a `theme-*` CSS class to your components/page containers and\nyou should do as well to your application HTML body element.\n\nIt will allow you to style raw HTML elements\n\n```css\nbody.theme-colorful {\n  font-family: // ...\n}\n\n.theme-colorful h1 {\n  font-family: // ...\n  font-size: // ...\n}\n```","ref":"theming.html#css-theming"},{"type":"extras","title":"Using a Registry - Theming components","doc":"This chapter explain how you can leverage on a `Registry` with the _function_ theming strategy.\n\nAn effective way to store the current theme setting so that it can be available to all your\ncomponents, but still have different values for different (concurrent) users is to associate it to\nthe current LiveView pid.\n\n`Registry` is a native Elixir module that handles decentralized storage, linked to specific\nprocesses. We will leverage on this to associate a theme to the current LiveView pid.\n\nFirst start a `Registry` from your `Application` module.\n\n```elixir\ndefmodule PhenixStorybook.Application do\n  def start(_type, _args) do\n    children = [\n      {Registry, keys: :duplicate, name: ThemeRegistry}\n    ]\n  end\nend\n```\n\nThen create a **LiveView Hook** that will fetch the theme from wherever it is relevant for your\napplication: database, user session, URL params... and store it in the `Registry` (it's working\nbecause the Hook is running under the same pid than the Liveview).\n\n```elixir\ndefmodule ThemeHook do\n  def on_mount(:default, params, _session, socket) do\n    theme = current_user_theme(socket, params)\n    Registry.register(ThemeRegistry, :theme, theme)\n    {:cont, socket}\n  end\nend\n```\n\nMount the hook in your `router`.\n\n```elixir\ndefmodule Router do\n  live_session :default, on_mount: [ThemeHook] do\n    scope \"/\" do\n      # ...\n    end\n  end\nend\n```\n\nWrite a helper module, to be used from your components to fetch the current theme from the\n`Registry` and merge it in the component's assigns.\n\n```elixir\ndefmodule ThemeHelpers do\n  def set_theme(assigns) do\n    pid_and_themes = Registry.lookup(ThemeRegistry, :theme)\n\n    case find_by_pid(pid_and_themes, self()) do\n      {_pid, theme} -> Map.put_new(assigns, :theme, theme)\n      _ -> raise(\"theme not found in registry\")\n    end\n  end\n\n  defp find_by_pid(pid_and_themes, current_pid) do\n    Enum.find(pid_and_themes, fn {pid, _} -> pid == current_pid end)\n  end\nend\n```","ref":"theming.html#using-a-registry"}],"content_type":"text/markdown","proglang":"elixir","producer":{"name":"ex_doc","version":"0.35.1"}}